# -*- coding: utf-8 -*-
from srf.views import APIView
from srf.generics import GenericAPIView
from srf import mixins
from srf.constant import DEFAULT_METHOD_MAP

__all__ = ('ViewSetMixin', 'ViewSet', 'GenericViewSet', 'ModelViewSet')


class ViewSetMixin():

    # @classmethod
    # def get_method_map(cls):
    #     methods = {}
    #     for method in ALL_METHOD:
    #         method = method.lower()
    #         if hasattr(cls, method):
    #             methods[method] = method
    #     return methods

    @classmethod
    def as_view(cls, method_map=DEFAULT_METHOD_MAP, *class_args, **class_kwargs):

        # 返回的响应方法闭包
        def view(request, *args, **kwargs):
            self = view.base_class(*class_args, **class_kwargs)
            view_method_map = {}
            for method, action in method_map.items():
                handler = getattr(self, action, None)
                if handler:
                    setattr(self, method, handler)
                    view_method_map[method] = action

            self.method_map = view_method_map
            self.methods = list(view_method_map.keys())
            self.request = request
            self.args = args
            self.kwargs = kwargs
            self.app = request.app
            return self.dispatch(request, *args, **kwargs)

        view.base_class = cls
        view.methods = list(method_map.keys())
        view.API_DOC_CONFIG = class_kwargs.get('API_DOC_CONFIG')  # 未来的API文档配置属性+
        view.__doc__ = cls.__doc__
        view.__module__ = cls.__module__
        view.__name__ = cls.__name__
        return view


class ViewSet(ViewSetMixin, APIView):
    """
    默认情况下，基本ViewSet类不提供任何操作。
    """
    pass


class GenericViewSet(ViewSetMixin, GenericAPIView):
    """
    GenericViewSet类默认不提供任何操作，但确实包含基本的通用视图行为集，
    例如 `get_object()` 和 `get_queryset()` 方法。
    """
    pass


class ModelViewSet(mixins.CreateModelMixin,
                   mixins.RetrieveModelMixin,
                   mixins.UpdateModelMixin,
                   mixins.DestroyModelMixin,
                   mixins.ListModelMixin,
                   GenericViewSet):
    """
    `create()`, `retrieve()`, `update()`, `partial_update()`, `destroy()`, `list()` actions.
    """
    pass



# -*- coding: utf-8 -*-

ALL_METHOD = {'GET', 'POST', 'PUT', 'PATCH', 'DELETE', 'HEAD', 'OPTIONS'}
DETAIL_METHOD_GROUP = {
    'dynamic_method': ['GET', 'PUT', 'DELETE', 'PATCH'],
    'static_method': ['POST', 'OPTION']
}
LIST_METHOD_GROUP = {
    'dynamic_method': ['PUT', 'DELETE', 'PATCH'],
    'static_method': ['GET', 'POST', 'OPTION']
}
DEFAULT_METHOD_MAP = {'get': 'get', 'post': 'post', 'put': 'put', 'patch': 'patch', 'delete': 'delete', 'head': 'head',
                      'options': 'options'}

LOOKUP_SEP = '__'

LIST_SERIALIZER_KWARGS = (
    'read_only',
    'write_only',
    'required',
    'allow_null',
    'default',
    'source',
    'validators',
    'error_messages',
    'label',
    'description',
    'instance',
    'data',
    'partial'
)
ALL_FIELDS = '__all__'



# -*- coding: utf-8 -*-
from tortoise import fields

from srf.fields import (CharField, IntegerField, BooleanField,
                    DecimalField, DateTimeField, DateField, FloatField, EnumChoiceField
                    )


def converts(*args):
    def _inner(func):
        func._converter_for = frozenset(args)
        return func

    return _inner


class ConverterException(Exception):
    pass


class ModelConverterBase(object):
    def __init__(self, nested_field_class, converters=None, ):
        self.nested_field_class = nested_field_class
        if not converters:
            converters = {}

        for name in dir(self):
            obj = getattr(self, name)
            if hasattr(obj, '_converter_for'):
                for classname in obj._converter_for:
                    converters[classname] = obj

        self.converters = converters


class ModelConverter(ModelConverterBase):
    """模型转换器"""

    def convert(self, serializer, model_field, **field_kwargs):
        model = serializer.Meta.model
        read_only = field_kwargs.get('read_only', False)
        write_only = field_kwargs.get('write_only', False)
        required = not model_field.null

        kwargs = {
            'read_only': read_only,
            'write_only': write_only,
            'allow_null': model_field.null,
            'description': model_field.description
        }
        if not read_only:
            kwargs['required'] = required

        if not isinstance(model_field, fields.relational.RelationalField):
            type_name = model_field.__class__.__name__
            if model_field.default is not None:
                kwargs['default'] = model_field.default
            converter = self.converters[type_name]
        else:
            type_name = model_field.__class__.__name__
            if hasattr(serializer.Meta, 'depth'):
                nested_depth = serializer.Meta.depth
            else:
                nested_depth = 10
            # kwargs['allow_empty'] = model_field.null
            kwargs['nested_depth'] = nested_depth
            converter = self.converters[type_name]
        kwargs.update(field_kwargs)

        return converter(model, model_field, **kwargs)

    @converts('CharField')
    def convert_charfield(self, model, model_field, *field_args, **field_kws):
        max_length = model_field.max_length
        if max_length is not None:
            field_kws['max_length'] = max_length
        return CharField(*field_args, **field_kws)

    @converts('UUIDField', 'JSONField', 'TextField')
    def convert_textfield(self, model, model_field, *field_args, **field_kws):
        return CharField(*field_args, **field_kws)

    @converts('IntField', 'BigIntField', 'SmallIntField')
    def convert_integerfield(self, model, model_field, *field_args, **field_kws):
        return IntegerField(*field_args, **field_kws)

    @converts('BooleanField')
    def convert_booleanfield(self, model, model_field, *field_args, **field_kws):
        return BooleanField(*field_args, **field_kws)

    @converts('CharEnumFieldInstance')
    def convert_charenumfield(self, model, model_field, *field_args, **field_kws):
        field_kws['enum_type'] = model_field.enum_type
        field_kws['value_type'] = str
        return EnumChoiceField(*field_args, **field_kws)

    @converts('IntEnumFieldInstance')
    def convert_intenumfield(self, model, model_field, *field_args, **field_kws):
        field_kws['enum_type'] = model_field.enum_type
        field_kws['value_type'] = int
        return EnumChoiceField(*field_args, **field_kws)

    @converts('DecimalField')
    def convert_decimalfield(self, model, model_field, *field_args, **field_kws):
        field_kws['max_digits'] = model_field.max_digits
        field_kws['decimal_places'] = model_field.decimal_places
        return DecimalField(*field_args, **field_kws)

    @converts('DatetimeField')
    def convert_datetimefield(self, model, model_field, *field_args, **field_kws):
        return DateTimeField(*field_args, **field_kws)

    @converts('DateField')
    def convert_datefield(self, model, model_field, *field_args, **field_kws):
        return DateField(*field_args, **field_kws)

    @converts('FloatField')
    def convert_floatfield(self, model, model_field, *field_args, **field_kws):
        return FloatField(*field_args, **field_kws)

    @converts('ManyToManyFieldInstance', 'ManyToManyRelation')
    def convert_manytomany(self, model, model_field, *field_args, **field_kws):
        """多对多"""
        nested_depth = field_kws.pop('nested_depth', 10)

        class NestedSerializer(self.nested_field_class):
            class Meta:
                model = model_field.related_model
                depth = nested_depth - 1
                fields = '__all__'

        return NestedSerializer(many=True)

    @converts('BackwardFKRelation', )
    def convert_backwardfkrelation(self, model, model_field, *field_args, **field_kws):
        """反向一对多"""
        nested_depth = field_kws.pop('nested_depth', 10)

        class NestedSerializer(self.nested_field_class):
            class Meta:
                model = model_field.related_model
                depth = nested_depth - 1
                fields = '__all__'

        return NestedSerializer(many=True)

    @converts('ForeignKeyFieldInstance')
    def convert_foreignkeyfieldinstance(self, model, model_field, *field_args, **field_kws):
        """正向多对一"""
        nested_depth = field_kws.pop('nested_depth', 10)

        class NestedSerializer(self.nested_field_class):
            class Meta:
                model = model_field.related_model
                depth = nested_depth - 1
                fields = '__all__'

        return NestedSerializer(**field_kws)

    @converts('OneToOneFieldInstance')
    def convert_onetoonefieldinstance(self, model, model_field, *field_args, **field_kws):
        """正向 一对一"""
        nested_depth = field_kws.pop('nested_depth', 10)

        class NestedSerializer(self.nested_field_class):
            class Meta:
                model = model_field.related_model
                depth = nested_depth - 1
                fields = '__all__'

        return NestedSerializer(**field_kws)

    @converts('BackwardOneToOneRelation')
    def convert_backwardonetoonerelation(self, model, model_field, *field_args, **field_kws):
        """反向一对一"""
        nested_depth = field_kws.pop('nested_depth', 10)

        class NestedSerializer(self.nested_field_class):
            class Meta:
                model = model_field.related_model
                depth = nested_depth - 1
                fields = '__all__'

        return NestedSerializer(**field_kws)



# -*- coding: utf-8 -*-
import datetime
import functools
import inspect
from decimal import Decimal
from urllib import parse

from tortoise.exceptions import IntegrityError

from srf.exceptions import APIException

_PROTECTED_TYPES = (
    type(None), int, float, Decimal, datetime.datetime, datetime.date, datetime.time,
)


class ObjectDict(dict):
    def __getattr__(self, key):
        return self[key]

    def __setattr__(self, key, value):
        if isinstance(value, dict):
            self[key] = ObjectDict(value)
        self[key] = value


def is_protected_type(obj):
    """确定对象实例是否为受保护的类型。
    受保护类型的对象在传递给时会原样保留
    force_str(strings_only = True)。
    """
    return isinstance(obj, _PROTECTED_TYPES)


def force_str(s, encoding='utf-8', strings_only=False, errors='strict'):
    """
    与smart_str()类似，除了将懒实例解析为
    字符串，而不是保留为惰性对象。
    如果strings_only为True，请不要转换（某些）非字符串类对象。
    """
    # 出于性能原因，请先处理常见情况。
    if issubclass(type(s), str):
        return s
    if strings_only and is_protected_type(s):
        return s
    try:
        if isinstance(s, bytes):
            s = str(s, encoding, errors)
        else:
            s = str(s)
    except UnicodeDecodeError as e:
        raise APIException('{value}出现解码错误'.format(value=s))
    return s


def replace_query_param(url, key, val):
    """
    给定一个URL和一个键/值对，在URL的查询参数中设置或替换一个项目，然后返回新的URL。
    """
    (scheme, netloc, path, query, fragment) = parse.urlsplit(force_str(url))
    query_dict = parse.parse_qs(query, keep_blank_values=True)
    query_dict[force_str(key)] = [force_str(val)]
    query = parse.urlencode(sorted(list(query_dict.items())), doseq=True)
    return parse.urlunsplit((scheme, netloc, path, query, fragment))


class IntegrityErrorHandel:
    def __init__(self, exc: IntegrityError):
        self.exc = exc
        self.message = str(exc)

    def parse_error_str(self):
        error = '发生错误：{}{}'
        if 'UNIQUE' in self.message:
            field_name = self.message.split('.')[-1]
            error = error.format(field_name, '已存在')
        return error

    def __str__(self):
        return self.parse_error_str()


async def run_awaitable(func, *args, **kwargs):
    if inspect.iscoroutinefunction(func):
        res = await func(*args, **kwargs)
    else:
        res = func(*args, **kwargs)
    return res


async def run_awaitable_val(value):
    if inspect.isawaitable(value):
        value = await value
    return value


def is_callable(obj):
    if not (inspect.isfunction(obj) or inspect.ismethod(obj) or isinstance(obj, functools.partial)):
        return False
    return True



import copy
from typing import Dict

from srf.exceptions import ValidationException, ValidatorAssertError

__all__ = ['BaseValidator', 'MaxLengthValidator', 'MinLengthValidator', 'MaxValueValidator', 'MinValueValidator']


class BaseValidator:
    """验证器基类
    所有通用验证器都需要继承本类，
    在调用 __call__ 时抛出 ValidationException 错误
    即代表验证失败
    """
    default_error_messages: Dict[str, str] = {

    }

    def __init__(self, error_messages: Dict[str, str] = None, code=None):
        self.error_messages = copy.copy(self.default_error_messages)
        if error_messages is not None:
            self.error_messages.update(copy.copy(error_messages))
        self.code = code

    def __call__(self, value, serializer=None):
        raise NotImplementedError('验证器必须重新定义 __call__()')

    def raise_error(self, key, **kws):
        msg = self.default_error_messages[key].format(**kws)
        raise ValidationException(msg, code=key)


class MaxLengthValidator(BaseValidator):
    default_error_messages: Dict[str, str] = {
        'max_length': '超出长度，最长支持{max_length}',
        'invalid': '无效的数据类型，数据类型只支持{datatypes}'
    }

    def __init__(self, max_length, **kwargs):
        if not isinstance(max_length, (int, float)):
            raise ValidatorAssertError('max_length的值只支持数值类型')
        self.max_length = max_length

        super(MaxLengthValidator, self).__init__(**kwargs)

    def __call__(self, value, serializer=None):
        if not isinstance(value, (str, list, dict, type)):
            self.raise_error('invalid', datatypes='str, list, dict, type')

        if len(value) > self.max_length:
            self.raise_error('max_length', max_length=self.max_length)


class MinLengthValidator(BaseValidator):
    default_error_messages: Dict[str, str] = {
        'min_length': '低于最低长度，最低为 {min_length}',
        'invalid': '无效的数据类型，数据类型只支持 {datatypes} '

    }

    def __init__(self, min_length, **kwargs):
        if not isinstance(min_length, (int, float)):
            raise ValidatorAssertError('min_length的值只支持数值类型')

        self.min_length = min_length
        super(MinLengthValidator, self).__init__(**kwargs)

    def __call__(self, value, serializer=None):
        if not isinstance(value, (str, list, dict, type)):
            self.raise_error('invalid', datatypes='str, list, dict, type')

        if len(value) < self.min_length:
            self.raise_error('min_length', min_length=self.min_length)


class MaxValueValidator(BaseValidator):
    default_error_messages: Dict[str, str] = {
        'max_value': '超出最大值，最大值支持到{max_value}',
        'invalid': '无效的数据类型，数据类型只支持{datatypes}'

    }

    def __init__(self, max_value, **kwargs):
        if not isinstance(max_value, (int, float)):
            raise ValidatorAssertError('max_value的值只支持数值类型')
        self.max_value = max_value
        super(MaxValueValidator, self).__init__(**kwargs)

    def __call__(self, value, serializer=None):
        if not isinstance(value, (int, float)):
            self.raise_error('invalid', datatypes='int, float')

        if value > self.max_value:
            self.raise_error('max_value', max_value=self.max_value)


class MinValueValidator(BaseValidator):
    default_error_messages: Dict[str, str] = {
        'min_value': '低于最小值，最小值至少要为{min_value}',
        'invalid': '无效的数据类型，数据类型只支持{datatypes}'
    }

    def __init__(self, min_value, **kwargs):
        if not isinstance(min_value, (int, float)):
            raise ValidatorAssertError('min_value的值只支持数值类型')
        self.min_value = min_value
        super(MinValueValidator, self).__init__(**kwargs)

    def __call__(self, value, serializer=None):
        if not isinstance(value, (int, float)):
            self.raise_error('invalid', datatypes='int, float')

        if value < self.min_value:
            self.raise_error('min_value', min_value=self.min_value)



# -*- coding: utf-8 -*-
from srf.utils import ObjectDict
from sanic.exceptions import InvalidUsage
from sanic.request import Request as SanicRequest


class SRFRequest(SanicRequest):
    def __init__(self, *args, **kwargs):
        super(SRFRequest, self).__init__(*args, **kwargs)
        self.user = None

    @property
    def data(self):
        try:
            data = self.json
        except InvalidUsage as exc:
            data = self.form
        data = {} if data is None else data
        return data


# -*- coding: utf-8 -*-
from tortoise.expressions import Q
from srf.constant import LOOKUP_SEP
__all__ = ('ORMAndFilter', 'ORMOrFilter')


class ORMAndFilter:
    """以And进行查询
        该类将直接得到 ORM_Filter
    """
    lookup_prefixes = {
        '^': 'istartswith',
        '$': 'iendswith',
        '>': 'gt',
        '<': 'lt',
        '>=': 'gte',
        '<=': 'lte',
        '=': 'contains',
        '@': 'icontains'
    }

    def __init__(self, request, view):
        """
        :param request: 当前请求
        :param view: 当前视图
        """
        self.view = view
        self.request = request

    def get_search_fields(self):
        """
        搜索字段是从视图获取的，但请求始终是
        传递给此方法。子类可以重写此方法以
        根据请求内容动态更改搜索字段。
        """
        return getattr(self.view, 'search_fields', None)

    @property
    def orm_filter(self):
        """
        根据定义的搜索字段过滤传入的queryset
        :return: Q object
        """
        orm_filters = []
        search_fields = self.get_search_fields()
        if not search_fields:
            return Q(*orm_filters)
        for search_field in search_fields:
            orm_filters.append(Q(**self.construct_orm_filter(search_field)))
        return Q(*orm_filters)

    def dismantle_search_field(self, search_field):
        """
        拆解带有特殊字符的搜索字段
        :param search_field: 搜索字段
        :return: (field_name, lookup_suffix)
        """
        lookup_suffix_keys = list(self.lookup_prefixes.keys())
        lookup_suffix = None
        field_name = search_field
        for lookup_suffix_key in lookup_suffix_keys:
            if lookup_suffix_key in search_field:
                lookup_suffix = self.lookup_prefixes[lookup_suffix_key]
                field_name = search_field[len(lookup_suffix_key):]
                return field_name, lookup_suffix
        return field_name, lookup_suffix

    def construct_orm_filter(self, search_field):
        """
        构造适用于orm的过滤参数
        :param search_field: 搜索字段
        :return:
        """
        field_name, lookup_suffix = self.dismantle_search_field(search_field)
        args = self.request.args

        if field_name not in args:
            return {}
        if lookup_suffix:
            orm_lookup = LOOKUP_SEP.join([field_name, lookup_suffix])
        else:
            orm_lookup = field_name
        return {orm_lookup: self.get_filter_value(field_name)}

    def get_filter_value(self, field_name):
        """
        根据字段名从请求中得到值
        :param field_name: 字段名
        :return:
        """
        values = self.request.args.get(field_name)
        return ''.join(values)


class ORMOrFilter(ORMAndFilter):
    """以And进行查询
        该类将直接得到 ORM_Filter
    """

    @property
    def orm_filter(self):
        """
        根据定义的搜索字段过滤传入的queryset
        :return:  Q object
        """
        orm_filters = []
        search_fields = self.get_search_fields()
        if not search_fields:
            return orm_filters
        for search_field in search_fields:
            orm_filters.append(Q(**self.construct_orm_filter(search_field)))
        return Q(*orm_filters, join_type=Q.OR)



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import List, Callable, Any

from sanic.log import logger
from sanic.response import json, HTTPResponse
from sanic.views import HTTPMethodView
from ujson import dumps
from tortoise.exceptions import IntegrityError
from tortoise.transactions import in_transaction

from srf.exceptions import APIException, ValidationException
from srf.permissions import BasePermission
from srf.status import RuleStatus, HttpStatus
from srf.utils import run_awaitable

__all__ = ('BaseView', 'APIView')


class BaseView:
    """只实现路由分发的基础视图
    在使用时应当开放全部路由 ALL_METHOD
    app.add_route('/test', BaseView.as_view(), 'test', ALL_METHOD)
    如需限制路由则在其他地方注明
    app.add_route('/test', BaseView.as_view(), 'test', ALL_METHOD)
    注意以上方法的报错是不可控的
    """

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

    @classmethod
    def as_view(cls, *class_args, **class_kwargs):
        # 返回的响应方法闭包
        async def view(request, *args, **kwargs):
            self = view.view_class(*class_args, **class_kwargs)

            self.request = request
            self.args = args
            self.kwargs = kwargs
            self.app = request.app
            return await self.dispatch(request, *args, **kwargs)

        view.view_class = cls
        view.API_DOC_CONFIG = class_kwargs.get('API_DOC_CONFIG')  # 未来的API文档配置属性+
        view.__doc__ = cls.__doc__
        view.__module__ = cls.__module__
        view.__name__ = cls.__name__
        return view

    async def dispatch(self, request, *args, **kwargs):
        """分发路由"""
        request.user = None
        method = request.method.lower()

        if not hasattr(self, method):
            return HTTPResponse('405请求方法错误', status=405)
        handler = getattr(self, method, None)
        return await run_awaitable(handler, request, *args, **kwargs)


class APIView(BaseView):
    """通用视图，可以基于其实现增删改查，提供权限套件"""
    authentication_classes = ()
    permission_classes = ()
    is_transaction = True

    async def dispatch(self, request, *args, **kwargs):
        """分发路由"""
        # request.user = None
        # setattr(request, 'user', None)
        method = request.method.lower()

        if not hasattr(self, method):
            return self.json_response(msg='发生错误：未找到%s方法' % method, errcode=400)
        handler = getattr(self, method, None)
        try:
            is_success = await self.initial(request, *args, **kwargs)
            if is_success is False:
                return self.json_response(msg='缺少token或token失效', errcode=4002)
            if self.is_transaction:
                async with in_transaction():
                    response = await run_awaitable(handler, request=request, *args, **kwargs)
            else:
                response = await run_awaitable(handler, request=request, *args, **kwargs)
        # except APIException as exc:
        #     response = self.handle_exception(exc)
        # except ValidationException as exc:
        #     response = self.error_json_response(exc.error_detail, '数据验证失败')
        # except AssertionError as exc:
        #     raise exc
        # except IntegrityError as exc:
        #     response = self.error_json_response(msg=str(exc), status=RuleStatus.STATUS_0_FAIL)
        except Exception as exc:
            # logger.error(f'--捕获未知错误--{exc}')
            response = self.handle_exception(exc)
            # msg = '发生致命的未知错误，请在服务器查看时间为{}的日志'.format(datetime.now().strftime('%F %T'))
            # response = self.json_response(msg=msg, errcode=400,
            #                               http_status=HttpStatus.HTTP_500_INTERNAL_SERVER_ERROR)
        return response

    def get_exception_handler(self):
        """
        Returns the exception handler that this view uses.
        """
        return getattr(self.app.ctx, "srf_exception_handler", None)

    def handle_exception(self, exc: APIException):
        exception_handler = self.get_exception_handler()
        if exception_handler is None:
            self.raise_uncaught_exception(exc)
        else:
            response = exception_handler(exc)
        response.exception = True
        return response

    def raise_uncaught_exception(self, exc):
        # if self.app.config.DEBUG:
        #     request = self.request
        #     renderer_format = getattr(request.accepted_renderer, 'format')
        #     use_plaintext_traceback = renderer_format not in ('html', 'api', 'admin')
        #     request.force_plaintext_errors(use_plaintext_traceback)
        raise exc


    def json_response(self, data=None, msg="请求成功", errcode=200,
                      http_status=HttpStatus.HTTP_200_OK):
        """
        Json 相应体
        :param data: 返回的数据主题
        :param msg: 前台提示字符串
        :param status: 前台约定状态，供前台判断是否成功
        :param http_status: Http响应数据
        :return:
        """
        if data is None:
            data = {}
        response_body = {
            'data': data,
            'code': errcode,
            'errcode': errcode,
            'errmsg': msg
        }
        return json(body=response_body, status=http_status, dumps=dumps)

    def success_json_response(self, data=None, msg="Success", **kwargs):
        """
        快捷的成功的json响应体
        :param data: 返回的数据主题
        :param msg: 前台提示字符串
        :return: json
        """
        return self.json_response(data=data, msg=msg, errcode=200)

    def error_json_response(self, data=None, msg="Fail", **kwargs):
        """
        快捷的失败的json响应体
        :param data: 返回的数据主题
        :param msg: 前台提示字符串
        :return: json
        """
        return self.json_response(data=data, msg=msg, errcode=400)

    def get_authenticators(self):
        """
        实例化并返回此视图可以使用的身份验证器列表
        """
        from srf.authentication import BaseAuthenticate
        authentications = []
        for auth in self.authentication_classes:
            if isinstance(auth, BaseAuthenticate):
                authentications.append(auth)
            else:
                authentications.append(auth())
        return authentications

    async def check_authentication(self, request):
        """
        检查权限 查看是否拥有权限，并在此处为Request.User 赋值
        :param request: 请求
        :return:
        """
        result = True
        for authenticators in self.get_authenticators():
            is_success = await authenticators.authenticate(request, self)
            if is_success is False:
                result = False
        return result

    def get_permissions(self):
        """
        实例化并返回此视图所需的权限列表
        """
        permissions = []
        for permission in self.permission_classes:
            if isinstance(permission, BasePermission):
                permissions.append(permission)
            else:
                permissions.append(permissions)
        return permissions

    async def check_permissions(self, request):
        """
        检查是否应允许该请求，如果不允许该请求，
        则在 has_permission 中引发一个适当的异常。
        :param request: 当前请求
        :return:
        """
        for permission in self.get_permissions():
            await permission.has_permission(request, self)

    async def check_object_permissions(self, request, obj):
        """
        检查是否应允许给定对象的请求, 如果不允许该请求，
        则在 has_object_permission 中引发一个适当的异常。
            常用于 get_object() 方法
        :param request: 当前请求
        :param obj: 需要鉴权的模型对象
        :return:
        """
        for permission in self.get_permissions():
            await permission.has_object_permission(request, self, obj)

    async def check_throttles(self, request):
        """
        检查范围频率。
        则引发一个 APIException 异常。
        :param request:
        :return:
        """
        pass

    async def initial(self, request, *args, **kwargs):
        """
        在请求分发之前执行初始化操作，用于检查权限及检查基础内容
        """
        is_success = await self.check_authentication(request)

        await self.check_permissions(request)
        await self.check_throttles(request)
        return is_success



# -*- coding: utf-8 -*-
from sanic import json
from ujson import dumps

from srf.views import APIView
import jwt
from jwt.exceptions import DecodeError, ExpiredSignatureError

from srf.exceptions import AuthenticationDenied
from srf.request import SRFRequest
from srf.status import HttpStatus
from srf.constant import ALL_METHOD


class BaseAuthenticate:
    def authenticate(self, request: SRFRequest, view: APIView, **kwargs):
        """验证权限并返回User对象"""
        pass


class BaseTokenAuthenticate(BaseAuthenticate):
    """基于Token的基础验证 JWT """
    token_key = 'authorization'

    async def authenticate(self, request: SRFRequest, view: APIView, **kwargs):
        """验证逻辑"""
        token = request.headers.get(self.token_key)
        if token is None:
            raise AuthenticationDenied('身份验证错误:请求头{}不存在'.format(self.token_key))
        token_secret = request.app.config.TOKEN_SECRET
        try:
            token_info = self.authentication_token(token, token_secret)
        except ExpiredSignatureError:
            raise AuthenticationDenied('登录已过期')
        except DecodeError:
            raise AuthenticationDenied('错误的Token')

        is_success = await self._authenticate(request, view, token_info, **kwargs)
        return is_success

    async def _authenticate(self, request: SRFRequest, view: APIView, token_info: dict, **kwargs):
        """主要处理逻辑"""
        pass

    def authentication_token(self, token, token_secret):
        """
        解包Token
        :param token: 口令
        :param token_secret: 解密秘钥
        :return:
        """
        token_info = jwt.decode(token, token_secret, algorithms=['HS256'])
        return token_info



# -*- coding: utf-8 -*-
from srf.exceptions import APIException
from srf.status import HttpStatus
from srf.utils import replace_query_param


class ORMPagination:
    """通用分页器"""
    page_size = 10
    page_query_param = 'page'
    page_size_query_param = 'size'
    max_page_size = 10000

    def __init__(self, request, view):
        """
        :param request: 当前请求
        :param view: 当前视图
        """
        self.view = view
        self.request = request

    def set_count(self, count):
        """
        设置集合
        :param count:
        :return:
        """
        self.count = count

    # @property
    # def next_page(self):
    #     """得到下一页的页码，不存在则返回None"""
    #     assert hasattr(self, 'count'), '必须先执行 `.set_count()` 函数才能使用.next_page'
    #     if self.query_page * self.page_size + self.page_size >= self.count:
    #         return None
    #     return self.query_page + 1
    #
    # @property
    # def next_link(self):
    #     """
    #     得到下一页的请求地址
    #     :return: None or String
    #     """
    #     page = self.next_page
    #     if not page:
    #         return None
    #     uri = self.request.server_path
    #     query_string = '?' + self.request.query_string
    #     query_string = replace_query_param(query_string, self.page_query_param, page)
    #     query_string = replace_query_param(query_string, self.page_size_query_param, self.page_size)
    #     return uri + query_string

    # @property
    # def previous_page(self):
    #     """得到上一页页码，不存在则返回None"""
    #     assert hasattr(self, 'count'), '必须先执行 `.set_count()` 函数才能使用.previous_page'
    #     if self.query_page * self.page_size <= 0:
    #         return None
    #     return self.query_page - 1

    # @property
    # def previous_link(self):
    #     """
    #     得到上一页的请求地址
    #     :return: None or String
    #     """
    #     if not self.previous_page:
    #         return None
    #     uri = self.request.server_path
    #     query_string = '?' + self.request.query_string
    #     query_string = replace_query_param(query_string, self.page_query_param, self.previous_page)
    #     query_string = replace_query_param(query_string, self.page_size_query_param, self.page_size)
    #     return uri + query_string

    @property
    def query_page(self):
        """得到页数"""
        try:
            page = int(self.request.args.get(self.page_query_param, 1))
        except ValueError as exc:
            raise APIException('发生错误的分页数据', http_status=HttpStatus.HTTP_400_BAD_REQUEST)
        if page < 1:
            page = 1
        return page

    @property
    def query_page_size(self):
        """得到页记录数"""
        try:
            page = int(self.request.args.get(self.page_size_query_param, self.page_size))
            if page > self.max_page_size:
                raise APIException('分页内容大小超出最大限制', http_status=HttpStatus.HTTP_400_BAD_REQUEST)
        except ValueError as exc:
            raise APIException('发生错误的分页数据', http_status=HttpStatus.HTTP_400_BAD_REQUEST)
        return page


# -*- coding: utf-8 -*-
import copy
import decimal
import re
import uuid
from datetime import timezone, timedelta, datetime, date, time
from enum import Enum
from typing import Any, List, Mapping
from uuid import UUID

from srf.utils import is_callable, run_awaitable, run_awaitable_val

from tortoise.exceptions import DoesNotExist

from srf.exceptions import ValidationException
from srf.validators import (
    MaxLengthValidator, MinLengthValidator, MaxValueValidator, MinValueValidator
)

REGEX_TYPE = type(re.compile(''))

__all__ = ('empty', 'SkipField', 'Field', 'CharField', 'IntegerField', 'FloatField', 'DecimalField', 'BooleanField',
           'DateTimeField', 'DateField', 'TimeField', 'ChoiceField', 'EnumChoiceField', 'SerializerMethodField')


class empty:
    """
    此类代表空，因为有些字段可以为 None
    所以需要一个可以替代 None 代表空变量
    """
    pass


class SkipField(Exception):
    pass


NOT_RAED_ONLY_AND_WRITE_ONLY = 'read_only 和 write_only 不能同时为True, 只能二选一'
NOT_RAED_ONLY_REQUIRED_ONLY = 'read_only 为 True 时 required 不能为True , 只能二选一'


class Field:
    """字段及序列化器基类
        required: 反序列化时是否必须存在，值限制写入时
        allow_null: 是否可以为 None 即存在当没值

    """
    _sort_counter = 0
    # 所有field都强制拥有的错误提示
    base_error_messages = {
        'required': '此字段为必填项，提交时必须携带',
        'null': '此字段不能为空',
    }
    default_error_messages = None

    default_validators = None

    def __init__(self, read_only=False, write_only=False, required=False, allow_null=False,
                 default=empty, source=None, validators=None, error_messages=None,
                 label=None, description=None):
        """
        字段及field的基类
        :param read_only: 只序列化
        :param write_only: 只反序列化
        :param required: 反序列化时必须存在此值
        :param allow_null: 反序列化时可以为 None, ''
        :param default: 默认值 可用于序列化和反序列化
        :param source: 序列化时值的来源
        :param validators: 反序列化时数据需要通过的验证
        :param error_messages: 出现错误时的自定义描述
        :param label: 字段标题
        :param description: 字段描述
        """
        assert not (read_only and write_only), NOT_RAED_ONLY_AND_WRITE_ONLY
        assert not (read_only and required), NOT_RAED_ONLY_REQUIRED_ONLY

        self._sort_counter = Field._sort_counter
        Field._sort_counter += 1

        self.read_only = read_only
        self.write_only = write_only
        self.required = required
        self.allow_null = allow_null
        self.default = default
        self.source = source
        self.label = label
        self.description = description

        self.validators = self.collect_validators([validators, self.default_validators])
        self.error_messages = self.collect_error_message(
            [self.base_error_messages, self.default_error_messages, error_messages])

        self.field_name = None
        self.parent = None

    def __new__(cls, *args, **kwargs):
        """
        当一个字段被实例化时，我们存储所使用的参数，
        这样我就可以在 __deepcopy__ 提供他们
        """
        instance = super().__new__(cls)
        instance._args = args
        instance._kwargs = kwargs
        return instance

    def __deepcopy__(self, memo):
        """
        当克隆字段时，我们使用参数实例化它
        最初创建时用的，而不是复制完整的状态。
        """
        args = [
            copy.deepcopy(item) if not isinstance(item, REGEX_TYPE) else item
            for item in self._args
        ]
        kwargs = {
            key: (copy.deepcopy(value, memo) if (key not in ('validators', 'regex')) else value)
            for key, value in self._kwargs.items()
        }
        return self.__class__(*args, **kwargs)

    def bind(self, field_name, parent):
        """
        提供给父级的绑定
        :param field_name:
        :param parent:
        :return:
        """
        import tortoise
        self.field_name = field_name
        self.parent = parent

        if self.source is None:
            self.source = self.field_name

        if self.source == '*':
            self.source_attrs = []
        else:
            self.source_attrs = self.source.split('.')

    def collect_error_message(self, error_messages_list: List[dict]) -> dict:
        """
        收集错误提示
        :param error_messages_list: 错误提示列表
        :return:
        """
        error_messages = {}
        for error_message in error_messages_list:
            if error_message is not None:
                error_messages.update(error_message)
        return error_messages

    def collect_validators(self, validators_list: List[list]) -> list:
        """
        收集所有验证器
        :param validators_list: 验证器列表
        :return:
        """
        validators = []
        for validator_list in validators_list:
            if validator_list is not None:
                for validator in validator_list:
                    if validator not in validators:
                        validators.append(validator)
        return validators

    def is_partial(self, root=None):
        """当请求为部分修改的时候,返回 True """
        if root is None:
            root = self.root
        return getattr(root, 'partial', False)

    async def external_to_internal(self, data: Any) -> Any:
        """对数据进行反序列化转换并返回"""
        raise NotImplementedError(
            '{cls}类的 .external_to_internal 方法必须重写'.format(cls=self.__class__.__name__, )
        )

    async def internal_to_external(self, data: Any) -> Any:
        """对数据进行序列化转换并返回"""
        raise NotImplementedError(
            '{cls}类的 .external_to_internal 方法必须重写'.format(cls=self.__class__.__name__, )
        )

    def get_external_value(self, data: Mapping) -> Any:
        """
        从传入的外部数据中得到值
        值用于输入验证
        :param data: *外部* 数据
        :return:
        """
        if not isinstance(data, Mapping):
            raise ValidationException('传入的数据为无效数据类型，仅支持字典类型'.format(self.field_name))
        if self.field_name not in data:
            if self.is_partial():
                return empty
            return self.default
        value = data.get(self.field_name)
        return value

    async def get_internal_value(self, instance: Any) -> Any:
        """
        从传入的内部数据中得到值
        值用于输出
        :param instance: *内部* 数据
        :return:
        """
        for attr in self.source_attrs:
            # 取值后可能为空，不能继续取
            if instance is None:
                return None
            # 不为空再取值
            try:
                if isinstance(instance, Mapping):
                    instance = instance[attr]
                else:
                    instance = getattr(instance, attr)
                    if is_callable(instance):
                        instance = await run_awaitable(instance)
                    else:
                        instance = await run_awaitable_val(instance)
            except DoesNotExist:
                return None
            except (KeyError, AttributeError) as exc:
                if self.default is not empty:
                    return self.default
                if self.allow_null:
                    return None
                if not self.required:
                    raise SkipField()
                raise type(exc)('在序列化过程中字段{field_name}未能成功序列化'.format(field_name=self.field_name))
        return instance

    def run_validators(self, data) -> None:
        """
        使用验证器验证传入的数据
        直接抛出错误
        :param data:
        :return: 无返回值
        """
        errors = []
        for validator in self.validators:
            try:
                validator(data, self)
            except ValidationException as exc:
                errors.extend(exc.error_detail)
        if errors:
            raise ValidationException(errors)

    def get_default(self):
        """得到默认值"""
        if self.default is empty or getattr(self.root, 'partial', False):
            raise SkipField()
        if callable(self.default):
            return self.default()
        return self.default

    def validate_empty_values(self, data):
        """
        判断是否为空值
        :param data: Any
        :return: (bool,Any) => (是否为空  ,data)
        """
        if self.read_only:
            return True, self.get_default()
        if data is empty:
            #
            if self.is_partial():
                raise SkipField()
            if self.required:
                self.raise_error('required')
            return True, self.get_default()

        if data is None:
            if not self.allow_null:
                self.raise_error('null')
            elif self.source == '*':
                return False, None
            return True, None
        return (False, data)

    async def run_validation(self, data):
        """执行验证"""
        (is_empty_value, data) = self.validate_empty_values(data)
        if is_empty_value:
            return data
        value = await self.external_to_internal(data)
        self.run_validators(value)
        return value

    @property
    def root(self):
        """
        得到字段的最高级父级
        """
        root = self
        while root.parent is not None:
            root = root.parent
        return root

    @property
    def context(self):
        """
        返回初始化时传递给根序列化程序的上下文。
        """
        return getattr(self.root, '_context', {})

    def raise_error(self, _key, **kwargs):
        """
        返回在 error_messages 中注册了的错误
        :param _key: 错误的键
        :param kwargs:
        :return:
        """
        try:
            msg = self.error_messages[_key]
        except KeyError:
            class_name = self.__class__.__name__
            msg = "在 {class_name} 类的 error_messages " \
                  "属性中未能找到 Key 为 {key} 的错误描述".format(class_name=class_name, key=_key)
            raise AssertionError(msg)
        message_string = msg.format(**kwargs)
        raise ValidationException(message_string, code=_key)

    def __str__(self):
        return super(Field, self).__str__() + self.field_name


class CharField(Field):
    """字符字段"""
    default_error_messages = {
        'invalid': '出现错误的数据类型，仅支持整字符类型',
        'max_length': '最长支持{max_length}个字符',
        'min_length': '至少要有{min_length}个字符',
    }

    def __init__(self, *args, **kwargs):
        self.max_length = kwargs.pop('max_length', None)
        self.min_length = kwargs.pop('min_length', None)
        self.trim_whitespace = kwargs.pop('trim_whitespace', True)
        super(CharField, self).__init__(*args, **kwargs)
        if self.max_length is not None:
            self.validators.append(MaxLengthValidator(max_length=self.max_length,
                                                      error_messages={'max_length': self.error_messages['max_length']}))
        if self.min_length is not None:
            self.validators.append(MinLengthValidator(min_length=self.min_length,
                                                      error_messages={'min_length': self.error_messages['min_length']}))

    async def external_to_internal(self, data: Any) -> Any:
        if isinstance(data, bool) or not isinstance(data, (str, int, float,)):
            self.raise_error('invalid')
        value = str(data)
        return value.strip() if self.trim_whitespace else value

    async def internal_to_external(self, data: Any) -> Any:
        return str(data)


class IntegerField(Field):
    """整数类型"""
    default_error_messages = {
        'invalid': '出现错误的数据类型，仅支持整数类型',
        'max_value': '仅支持小于{max_value}的整数',
        'min_value': '仅支持大于{min_value}的整数',
        'max_string_length': '仅支持转换长度小于{max_string_length}的整数字符串',
    }
    re_decimal = re.compile(r'\.0*\s*$')
    MAX_STRING_LENGTH = 1000

    def __init__(self, max_value=None, min_value=None, *args, **kwargs):
        self.max_value = max_value
        self.min_value = min_value
        super(IntegerField, self).__init__(*args, **kwargs)
        if self.max_value is not None:
            self.validators.append(MaxValueValidator(max_value=self.max_value,
                                                     error_messages={'max_value': self.error_messages['max_value']}))
        if self.min_value is not None:
            self.validators.append(MinValueValidator(min_value=self.min_value,
                                                     error_messages={'min_value': self.error_messages['min_value']}))

    async def external_to_internal(self, data: Any):
        if isinstance(data, str) and len(data) > self.MAX_STRING_LENGTH:
            self.raise_error('max_string_length', max_string_length=self.MAX_STRING_LENGTH)
        try:
            data = int(self.re_decimal.sub('', str(data)))
        except (ValueError, TypeError):
            self.raise_error('invalid')
        return data

    async def internal_to_external(self, data: Any):
        return int(data)


class FloatField(IntegerField):
    """浮点类型"""
    default_error_messages = {
        'invalid': '出现错误的数据类型{data_type}，仅支持浮点类型',
        'max_value': '仅支持小于{max_value}浮点',
        'min_value': '仅支持大于{min_value}浮点',
        'max_string_length': '仅支持转换长度小于{max_string_length}的浮点字符串',
    }
    MAX_STRING_LENGTH = 1000

    async def external_to_internal(self, data: Any):
        if isinstance(data, bool):
            self.raise_error('invalid', data_type=type(data).__name__)
        if isinstance(data, str) and len(data) > self.MAX_STRING_LENGTH:
            self.raise_error('max_string_length', max_string_length=self.MAX_STRING_LENGTH)
        try:
            return float(data)
        except (TypeError, ValueError):
            self.raise_error('invalid', data_type=type(data).__name__)

    async def internal_to_external(self, data: Any):
        return float(data)


class DecimalField(Field):
    """十进制类型"""
    default_error_messages = {
        'invalid': '出现错误的数据类型，仅支持Decimal十进制类型',
        'max_value': '仅支持小于{max_value}Decimal十进制类型',
        'min_value': '仅支持大于{min_value}Decimal十进制类型',
        'max_string_length': '仅支持转换长度小于{max_string_length}的Decimal十进制字符串',
        'max_digits': '确保总数不超过{max_digits}个数字。',
        'max_decimal_places': '确保不超过{max_decimal_places}个小数位。',
        'max_whole_digits': '确保小数点前的位数不超过{max_whole_digits}个。',
    }
    MAX_STRING_LENGTH = 1000

    def __init__(self, max_digits, decimal_places, coerce_to_string=False, max_value=None, min_value=None,
                 rounding=None, *args, **kwargs):
        """
        整数位数 = max_digits - decimal_places
        :param max_digits: 数字允许的最大位数
        :param decimal_places: 小数的最大位数
        :param coerce_to_string:
        :param max_value:
        :param min_value:
        :param rounding:
        :param args:
        :param kwargs:
        """
        self.max_digits = max_digits
        self.decimal_places = decimal_places
        self.coerce_to_string = coerce_to_string
        self.max_value = max_value
        self.min_value = min_value
        self.rounding = rounding
        if self.max_digits is not None and self.decimal_places is not None:
            self.max_whole_digits = self.max_digits - self.decimal_places
        else:
            self.max_whole_digits = None
        super(DecimalField, self).__init__(*args, **kwargs)
        if self.max_value is not None:
            self.validators.append(MaxValueValidator(max_value=self.max_value,
                                                     error_messages={'max_value': self.error_messages['max_value']}))
        if self.min_value is not None:
            self.validators.append(MinValueValidator(min_value=self.min_value,
                                                     error_messages={'min_value': self.error_messages['min_value']}))

    async def external_to_internal(self, data: Any):
        data = str(data).strip()

        if len(data) > self.MAX_STRING_LENGTH:
            self.raise_error('max_string_length', max_string_length=self.MAX_STRING_LENGTH)
        try:
            data = decimal.Decimal(data)
        except decimal.DecimalException:
            self.raise_error('invalid')

        if data.is_nan():
            self.raise_error('invalid')

        # 检查无穷大和负无穷大。
        if data in (decimal.Decimal('Inf'), decimal.Decimal('-Inf')):
            self.raise_error('invalid')

        return self.quantize(self.validate_precision(data))

    async def internal_to_external(self, data: Any):
        if not isinstance(data, decimal.Decimal):
            data = decimal.Decimal(str(data).strip())

        quantized = self.quantize(data)

        if not self.coerce_to_string:
            return quantized
        return ('{:%sf}' % self.decimal_places).format(quantized)

    def validate_precision(self, value):
        """
        确保数字中的位数不超过max_digits，并且小数点后超过十进制的位数。
        覆盖此方法以禁用输入的精度验证值或以您需要的任何方式增强它。
        """
        sign, digittuple, exponent = value.as_tuple()

        if exponent >= 0:
            # 1234500.0
            total_digits = len(digittuple) + exponent
            whole_digits = total_digits
            decimal_places = 0
        elif len(digittuple) > abs(exponent):
            # 123.45
            total_digits = len(digittuple)
            whole_digits = total_digits - abs(exponent)
            decimal_places = abs(exponent)
        else:
            # 0.001234
            total_digits = abs(exponent)
            whole_digits = 0
            decimal_places = total_digits

        if self.max_digits is not None and total_digits > self.max_digits:
            self.raise_error('max_digits', max_digits=self.max_digits)
        if self.decimal_places is not None and decimal_places > self.decimal_places:
            self.raise_error('max_decimal_places', max_decimal_places=self.decimal_places)
        if self.max_whole_digits is not None and whole_digits > self.max_whole_digits:
            self.raise_error('max_whole_digits', max_whole_digits=self.max_whole_digits)
        return value

    def quantize(self, value):
        """
        将十进制值量化为配置的精度。
        """
        if self.decimal_places is None:
            return value

        context = decimal.getcontext().copy()
        if self.max_digits is not None:
            context.prec = self.max_digits
        return value.quantize(
            decimal.Decimal('.1') ** self.decimal_places,
            rounding=self.rounding,
            context=context
        )


class BooleanField(Field):
    """布尔值类型"""
    default_error_messages = {
        'invalid': '出现错误的数据类型，{value}不是有效的布尔值',
    }
    TRUE_VALUES = {
        't', 'T',
        'y', 'Y', 'yes', 'YES',
        'true', 'True', 'TRUE',
        'on', 'On', 'ON',
        '1', 1,
        True
    }
    FALSE_VALUES = {
        'f', 'F',
        'n', 'N', 'no', 'NO',
        'false', 'False', 'FALSE',
        'off', 'Off', 'OFF',
        '0', 0, 0.0,
        False
    }
    NULL_VALUES = {'null', 'Null', 'NULL', '', None}

    async def external_to_internal(self, data: Any) -> Any:
        try:
            if data in self.TRUE_VALUES:
                return True
            elif data in self.FALSE_VALUES:
                return False
            elif data in self.NULL_VALUES and self.allow_null:
                return None
        except TypeError:
            self.raise_error('invalid', value=data)

    async def internal_to_external(self, data: Any) -> Any:
        if data in self.TRUE_VALUES:
            return True
        elif data in self.FALSE_VALUES:
            return False
        if data in self.NULL_VALUES and self.allow_null:
            return None
        return bool(data)


class DateTimeField(Field):
    """日期时间类型"""
    default_error_messages = {
        'invalid': '错误的数据类型{data_type}, 不能转换为字符格式',
        'convert': '日期转换异常，请确认日期格式符合 %Y-%m-%d %H:%M:%S 规则',
        'date': '需要的是日期时间格式而不是日期格式',
        'overflow': '时间超出范围'
    }

    def __init__(self, output_format='%Y-%m-%d %H:%M:%S', input_format='%Y-%m-%d %H:%M:%S',
                 set_timezone: timezone = None, *args, **kwargs):
        self.output_format = output_format
        self.input_format = input_format
        if set_timezone is not None:
            self.set_timezone = set_timezone
        else:
            self.set_timezone = self.get_default_timezone()
        super(DateTimeField, self).__init__(*args, **kwargs)

    def get_default_timezone(self):
        """设置默认时区为北京时间"""
        return timezone(timedelta(hours=8))

    def enforce_timezone(self, value):
        """强制设置一个时区"""
        return value.astimezone(self.set_timezone)

    async def external_to_internal(self, data: Any) -> Any:
        if not isinstance(data, (str, date, datetime)):
            self.raise_error('convert')
        if type(data) == date:
            self.raise_error('date')
        if isinstance(data, str):
            try:
                data = datetime.strptime(data, self.input_format)
            except (ValueError, TypeError):
                self.raise_error('convert')
        if type(data) == datetime:
            data = self.enforce_timezone(data)
        return data

    async def internal_to_external(self, data: Any) -> Any:
        if isinstance(data, str):
            try:
                data = datetime.strptime(data, self.input_format)
            except (ValueError, TypeError):
                self.raise_error('convert')
        if isinstance(data, datetime):
            return data.strftime(self.output_format)
        self.raise_error('invalid', data_type=type(data).__name__)


class DateField(Field):
    """日期字段"""
    default_error_messages = {
        'invalid': '出现错误的数据类型，{value}不是有效的日期时间类型',
        'datetime': '需要的是日期格式而不是日期时间格式',
    }

    def __init__(self, output_format='%Y-%m-%d', input_format='%Y-%m-%d', *args, **kwargs):
        self.output_format = output_format
        self.input_format = input_format
        super(DateField, self).__init__(*args, **kwargs)

    async def external_to_internal(self, data: Any) -> Any:
        if isinstance(data, str):
            try:
                data = datetime.strptime(data, self.input_format).date()
            except (ValueError, TypeError):
                self.raise_error('invalid', value=data)
        if isinstance(data, datetime):
            self.raise_error('datetime')
        if isinstance(data, date):
            return data
        self.raise_error('invalid', value=data)

    async def internal_to_external(self, data: Any) -> Any:
        if isinstance(data, str):
            try:
                data = datetime.strptime(data, self.input_format).date()
            except (ValueError, TypeError):
                self.raise_error('invalid', value=data)
        if isinstance(data, date):
            return data.strftime(self.output_format)
        self.raise_error('invalid', value=data)


class TimeField(Field):
    """时间字段"""
    default_error_messages = {
        'invalid': '出现错误的数据类型，{value}不是有效的日期时间类型',
        'format': '时间格式错误，需要格式为 %H:%M:%S ',
        'date': '需要的是时间格式而不是日期格式',
        'datetime': '需要的是时间格式而不是日期时间格式',
    }

    def __init__(self, output_format='%H:%M:%S', input_format='%H:%M:%S', *args, **kwargs):
        self.output_format = output_format
        self.input_format = input_format
        super(TimeField, self).__init__(*args, **kwargs)

    async def external_to_internal(self, data: Any) -> Any:
        if isinstance(data, str):
            try:
                data = datetime.strptime(data, self.input_format).time()
                return data
            except (ValueError, TypeError):
                self.raise_error('format')
        if isinstance(data, datetime):
            self.raise_error('datetime')
        if isinstance(data, time):
            return data
        if isinstance(data, date):
            self.raise_error('date')
        self.raise_error('invalid', value=type(data))

    async def internal_to_external(self, data: Any) -> Any:
        if isinstance(data, str):
            try:
                data = datetime.strptime(data, self.input_format).time()
            except (ValueError, TypeError):
                self.raise_error('format')
        if isinstance(data, datetime):
            data = data.time()
        if isinstance(data, time):
            return data.strftime(self.output_format)
        self.raise_error('invalid', value=data)


class ChoiceField(Field):
    """限定可选的字段"""
    default_error_messages = {
        'invalid': '出现错误的数据类型，{value}不是有效的日期时间类型',
        'key': '错误选项{key}',
    }

    def __init__(self, choices, *args, **kwargs):
        """
        :param choices: (('key','value'),('key','value'),)
        :param args:
        :param kwargs:
        """
        self.choices = choices
        super(ChoiceField, self).__init__(*args, **kwargs)

    async def external_to_internal(self, data: Any) -> Any:
        return self.choices_get_value_by_key(data)

    async def internal_to_external(self, data: Any) -> Any:
        return self.choices_get_value_by_key(data)

    def choices_get_value_by_key(self, key):
        """得到字符串"""
        if self.check_key_choices(key):
            choices_dict = self.get_choices()
            value = choices_dict[key]
            return value
        self.raise_error('key', key=key)

    def check_key_choices(self, key):
        choices_dict = self.get_choices()
        return key in choices_dict

    def get_choices(self) -> dict:
        choices = {key: value for key, value in self.choices}
        return choices


class EnumChoiceField(Field):
    """枚举类型字段"""

    def __init__(self, enum_type, value_type, *args, **kwargs):
        """
        :param enum_type: 枚举类
        :param value_type: 枚举类
        :param value_map: 值的映射对象
        :param args:
        :param kwargs:
        """
        self.enum_type = enum_type
        self.value_type = value_type
        super(EnumChoiceField, self).__init__(*args, **kwargs)

    async def external_to_internal(self, data: Any) -> Any:
        return self.enum_type(self.value_type(data)) if data is not None else None

    async def internal_to_external(self, data: Any) -> Any:
        if isinstance(data, Enum):
            return self.value_type(data.value)
        if isinstance(data, self.value_type):
            return self.value_type(self.enum_type(data).value)
        return self.value_type(data)


class SerializerMethodField(Field):
    """
    一个只读字段，可通过在父序列化器类。调用的方法将具有以下形式
    “ get_ {field_name}”，并且应采用单个参数，即
    对象被序列化。
    For example:

    class ExampleSerializer(self):
        extra_info = SerializerMethodField()

        def get_extra_info(self, obj):
            return ...  # Calculate some data to return.
    """

    def __init__(self, method_name=None, **kwargs):
        self.method_name = method_name
        kwargs['source'] = '*'
        kwargs['read_only'] = True
        if kwargs.get('required'):
            assert 'SerializerMethodField 为只读字段，不能反序列化'

        super().__init__(**kwargs)

    def bind(self, field_name, parent):
        # The method name defaults to `get_{field_name}`.
        if self.method_name is None:
            self.method_name = 'get_{field_name}'.format(field_name=field_name)

        super().bind(field_name, parent)

    async def internal_to_external(self, data: Any) -> Any:
        method = getattr(self.parent, self.method_name)
        return await run_awaitable(method, data)

    def external_to_internal(self, data: Any) -> Any:
        raise ValidationException('SerializerMethodField 不支持反序列化')



# -*- coding: utf-8 -*-
import copy
import inspect
from collections import OrderedDict
from typing import Any, Mapping

from tortoise import fields as tortoise_fields
from tortoise.queryset import ValuesQuery, ValuesListQuery

from srf.utils import run_awaitable, run_awaitable_val
from srf.converter import ModelConverter
from srf.fields import (
    empty, SkipField, Field, TimeField, DateTimeField, SerializerMethodField,
    CharField, ChoiceField, DateField, DecimalField, FloatField,
    IntegerField,
)
from srf.exceptions import ValidationException
from srf.helpers import BindingDict
from srf.constant import LIST_SERIALIZER_KWARGS, ALL_FIELDS


def set_value(dictionary, keys, value):
    """
    set_value({'a': 1}, [], {'b': 2}) -> {'a': 1, 'b': 2}
    set_value({'a': 1}, ['x'], 2) -> {'a': 1, 'x': 2}
    set_value({'a': 1}, ['x', 'y'], 2) -> {'a': 1, 'x': {'y': 2}}
    """
    if not keys:
        dictionary.update(value)
        return

    for key in keys[:-1]:
        if key not in dictionary:
            dictionary[key] = {}
        dictionary = dictionary[key]

    dictionary[keys[-1]] = value


class BaseSerializer(Field):
    """序列化器
    .instance ->
        .get_internal_value ->
            .internal_to_external() ->
                .data

    .install_data -> .get_external_value -> .external_to_internal() -> .validated_data
    """

    def __init__(self, instance=None, data=empty, **kwargs):
        self.instance = instance
        if data is not empty:
            self.initial_data = data
        self.partial = kwargs.pop('partial', False)
        self._context = kwargs.pop('context', {})
        kwargs.pop('many', None)
        super().__init__(**kwargs)

    def __new__(cls, *args, **kwargs):
        if kwargs.pop('many', False):
            return cls.many_init(*args, **kwargs)
        return super().__new__(cls, *args, **kwargs)

    @classmethod
    def many_init(cls, *args, **kwargs):
        """初始化多值组件"""
        # allow_empty = kwargs.pop('allow_empty', None)
        child_serializer = cls(*args, **kwargs)
        list_kwargs = {
            'child': child_serializer,
        }
        # if allow_empty is not None:
        #     list_kwargs['allow_empty'] = allow_empty
        list_kwargs.update({
            key: value for key, value in kwargs.items()
            if key in LIST_SERIALIZER_KWARGS
        })
        meta = getattr(cls, 'Meta', None)
        list_serializer_class = getattr(meta, 'list_serializer_class', ListSerializer)
        return list_serializer_class(*args, **list_kwargs)

    async def external_to_internal(self, data: Any) -> Any:
        """对数据进行序列化转换并返回"""
        raise NotImplementedError(
            '{cls}类的 .external_to_internal 方法必须重写'.format(cls=self.__class__.__name__, )
        )

    async def internal_to_external(self, data: Any) -> Any:
        """对数据进行反序列化转换并返回"""
        raise NotImplementedError(
            '{cls}类的 .external_to_internal 方法必须重写'.format(cls=self.__class__.__name__, )
        )

    async def validate(self, attr):
        return attr

    async def run_validation(self, data):
        value = await super(BaseSerializer, self).run_validation(data)
        value = self.validate(value)
        value = await run_awaitable_val(value)
        return value

    async def update(self, instance, validated_data):
        raise NotImplementedError('`update()` 必须实现.')

    async def create(self, validated_data):
        raise NotImplementedError('`create()` 必须实现.')

    async def save(self, **kwargs):
        assert hasattr(self, '_errors'), (
            '您必须先调用`.is_valid()`，然后再调用`.save()`。'
        )

        assert not self.errors, (
            '您不能在未通过效验的序列化器上调用`.save()`。'
        )

        # Guard against incorrect use of `serializer.save(commit=False)`
        assert 'commit' not in kwargs, (
            "`commit`不是`save()`方法的有效关键字参数。"
            "如果需要在提交数据库之前访问数据，则"
            "而是检查`serializer.validated_data`."
            "如果您还可以将其他关键字参数传递给`save()`，"
            "需要在保存的模型实例上设置额外的属性。"
            "例如：'serializer.save(owner = request.user)'。"
        )

        assert not hasattr(self, '_data'), (
            '访问`serializer.data`后，您不能调用`.save()`。'
            '如果需要在提交数据库之前访问数据，请访问'
            '`serializer.validated_data`'
        )

        validated_data = dict(list(self.validated_data.items()) + list(kwargs.items()))
        if self.instance is not None:
            validated_data, self.instance = await self.before_update(validated_data, self.instance)
            self.instance = await self.update(self.instance, validated_data)
            assert self.instance is not None, (
                '`update()` 没有返回对象实例。'
            )
        else:
            validated_data, self.instance = await self.before_create(validated_data, self.instance)
            self.instance = await self.create(validated_data)
            assert self.instance is not None, (
                '`create()` 没有返回对象实例。'
            )
        return self.instance

    async def before_update(self, validated_data, instance):
        return validated_data, instance

    async def before_create(self, validated_data, instance):
        return validated_data, instance

    async def is_valid(self, raise_exception=False):
        assert hasattr(self, 'initial_data'), (
            '无法调用`.is_valid()`，因为'
            '类实例化时没有传入`data =`关键字参数'
        )

        if not hasattr(self, '_validated_data'):
            try:
                self._validated_data = await self.run_validation(self.initial_data)
            except ValidationException as exc:
                self._validated_data = {}
                self._errors = exc.error_detail
            else:
                self._errors = {}

        if self._errors and raise_exception:
            raise ValidationException(self.errors)
        return not bool(self._errors)

    @property
    async def data(self):
        """对外呈现的数据"""
        assert not self.instance is None, '调用 .data 必须先传入 instance= '
        if not hasattr(self, '_data'):
            self._data = await self.internal_to_external(self.instance)
        return self._data

    @property
    def errors(self):
        """对外呈现验证过程中出现的错误"""
        if not hasattr(self, '_errors'):
            msg = '你必须先执行 .is_valid() 方法才能调用 .errors'
            raise AssertionError(msg)
        return self._errors

    @property
    def validated_data(self):
        """对内使用的验证后的数据"""
        if not hasattr(self, '_validated_data'):
            msg = '你必须先执行 .is_valid() 方法才能调用 .validated_data'
            raise AssertionError(msg)
        return self._validated_data


class SerializerMetaclass(type):

    @classmethod
    def _get_declared_fields(cls, bases, attrs):
        fields = [(field_name, attrs.pop(field_name))
                  for field_name, obj in list(attrs.items())
                  if isinstance(obj, Field)]
        fields.sort(key=lambda x: x[1]._sort_counter)

        known = set(attrs)

        def visit(name):
            known.add(name)
            return name

        base_fields = []
        for base in bases:
            if hasattr(base, '_declared_fields'):
                for name, field in base._declared_fields.items():
                    if name not in known:
                        base_fields.append((visit(name), field))

        return OrderedDict(base_fields + fields)

    def __new__(cls, name, bases, attrs):
        attrs['_declared_fields'] = cls._get_declared_fields(bases, attrs)
        return super().__new__(cls, name, bases, attrs)


class Serializer(BaseSerializer, metaclass=SerializerMetaclass):
    default_error_messages = {
        'invalid': '无效数据。应该是字典，但是得到了{datatype}'
    }

    @property
    def fields(self):
        """
        单个格式为 {field_name: field_instance}.
        fields 是动态加载的 避免在导入时出现意想不到的错误
        """
        # like drf
        fields = BindingDict(self)
        for key, value in self.get_fields().items():
            fields[key] = value
        return fields

    @property
    def _writable_fields(self):
        for field in self.fields.values():
            if not field.read_only:
                yield field

    @property
    def _readable_fields(self):
        for field in self.fields.values():
            if not field.write_only:
                yield field

    def get_fields(self) -> dict:
        """
        得到所有fields
        :return:
        """
        return copy.deepcopy(self._declared_fields)

    # 序列化
    async def internal_to_external(self, data: Any) -> Any:
        """
        内转外
        :param data:
        :return:

        """
        res = OrderedDict()
        fields = self._readable_fields
        for field in fields:
            method = getattr(self, 'output_{}'.format(field.field_name), None)
            try:
                value = await field.get_internal_value(data)
            except SkipField:
                continue
            if value is not None:
                value = await field.internal_to_external(value)
            if method:
                value = await run_awaitable(method, value, data)
            res[field.field_name] = value
        return res

    #  反序列化
    async def external_to_internal(self, data: Any) -> Any:
        """
        外转内
        :param data:
        :return:
        """
        res = OrderedDict()
        errors = OrderedDict()
        fields = self._writable_fields
        for field in fields:
            validate_method = getattr(self, 'validate_' + field.field_name, None)
            try:
                primitive_value = field.get_external_value(data)
                validated_value = await field.run_validation(primitive_value)
                if validate_method is not None:
                    validated_value = await run_awaitable(validate_method, validated_value, data)
            except ValidationException as exc:
                errors[field.field_name] = exc.error_detail
            except SkipField:
                pass
            else:
                set_value(res, field.source_attrs, validated_value)
        if errors:
            raise ValidationException(errors)
        return res

    async def validate(self, attrs):
        return attrs

    def __iter__(self):
        for field in self.fields.values():
            yield self[field.field_name]

    def __getitem__(self, key):
        field = self.fields[key]
        value = self.data.get(key)
        error = self.errors.get(key) if hasattr(self, '_errors') else None
        return {
            'field': field,
            'value': value,
            'error': error,
        }


class ListSerializer(BaseSerializer):
    child = None
    many = True

    default_error_messages = {
        'not_a_list': '预期项目列表，但类型为“ {input_type}”。',
        'null': '此列表不能为空。'
    }

    def __init__(self, *args, **kwargs):
        self.child = kwargs.pop('child', copy.deepcopy(self.child))
        # self.allow_empty = kwargs.pop('allow_empty', True)
        assert self.child is not None, '`child` 是必填参数。'
        assert not inspect.isclass(self.child), '`child` 尚未实例化。'
        super().__init__(*args, **kwargs)
        self.child.bind(field_name='', parent=self)

    async def get_internal_value(self, instance: Any) -> Any:
        """目的得到值"""
        for attr in self.source_attrs:
            if isinstance(instance, Mapping):
                instance = instance.get(attr, [])
                if not isinstance(instance, list):
                    instance = [instance]
            else:
                instance = getattr(instance, attr)
        return instance

    async def internal_to_external(self, data: Any) -> Any:
        """
        内转外
        :param data:
        :return:
        """
        if not inspect.isawaitable(data):
            iterable = data
        else:
            if isinstance(data, (ValuesQuery, ValuesListQuery)):
                iterable = await data
            else:
                iterable = await data.all()

        return [
            await self.child.internal_to_external(item) for item in iterable
        ]

    async def external_to_internal(self, data: Any) -> Any:
        """
        外转内
        :param data:
        :return:
        """
        if not isinstance(data, list):
            raise self.raise_error('not_a_list', input_type=type(data).__name__)

        if not self.allow_null and len(data) == 0:
            raise self.raise_error('null')

        ret = []
        errors = []

        for item in data:
            try:
                value = await self.child.run_validation(item)
            except ValidationException as exc:
                errors.append(exc.error_detail)
            else:
                ret.append(value)
                errors.append({})
        if any(errors):
            raise ValidationException(errors)
        return ret

    async def run_validation(self, data=empty):
        """
        我们覆盖默认的`run_validation`，因为验证由验证者执行，
        而.validate（）方法应使用“non_fields_error”键被强制为错误字典。
        """
        (is_empty_value, data) = self.validate_empty_values(data)
        if is_empty_value:
            return data
        value = await self.external_to_internal(data)
        self.run_validators(value)
        value = await self.validate(value)
        return value

    async def update(self, instance, validated_data):
        raise NotImplementedError(
            '当 many=True 时，有些序列化器不支持更新操作，'
            '所以在必须使用时请继承ListSerializer并覆盖'
            '`.update()`，不提供默认实现方式')

    async def create(self, validated_data):
        return [
            await self.child.create(attrs) for attrs in validated_data
        ]

    async def save(self, **kwargs):
        """
        保存实例
        """
        # 防止错误使用`serializer.save(commit = False)`
        assert 'commit' not in kwargs, (
            "`commit`不是`save()`方法的有效关键字参数。"
            "如果需要在提交数据库之前访问数据，则"
            "而是检查`serializer.validated_data`."
            "如果您还可以将其他关键字参数传递给`save()`，"
            "需要在保存的模型实例上设置额外的属性。"
            "例如：'serializer.save(owner = request.user)'。"
        )

        validated_data = [
            dict(list(attrs.items()) + list(kwargs.items()))
            for attrs in self.validated_data
        ]

        if self.instance is not None:
            self.instance = await self.update(self.instance, validated_data)
            assert self.instance is not None, (
                '`update()` 没有返回对象实例。'
            )
        else:
            self.instance = await self.create(validated_data)
            assert self.instance is not None, (
                '`create()` 没有返回对象实例。'
            )

        return self.instance


class ModelSerializer(Serializer):
    """
        class Meta:
            model = None
            depth = 0
            extra_kwargs = {}
            fields = ()  # 冲突，不能与exclude共存
            exclude = ()  # 冲突，不能与fields共存
            read_only_fields = ()  # 字段与write_only_fields冲突
            write_only_fields = ()  # 字段与read_only_fields冲突

    """

    @property
    def fields(self):
        """
        单个格式为 {field_name: field_instance}.
        fields 是动态加载的 避免在导入时出现意想不到的错误
        """
        assert hasattr(self, 'Meta'), (
            '{serializer_class} 类没有 "Meta" 属性'.format(
                serializer_class=self.__class__.__name__
            )
        )
        assert hasattr(self.Meta, 'model'), (
            '{serializer_class} 类没有 "Meta.model" 属性'.format(
                serializer_class=self.__class__.__name__
            )
        )
        if self.Meta.model._meta.abstract:
            raise ValueError('不能将ModelSerializer与抽象模型一起使用。')

        declared_fields = copy.deepcopy(self._declared_fields)
        model = getattr(self.Meta, 'model')
        depth = getattr(self.Meta, 'depth', 10)
        if depth is not None:
            assert depth >= 0, "'depth' may not be negative."
            assert depth <= 10, "'depth' may not be greater than 10."

        converter = ModelConverter(ModelSerializer)
        model_original_fields = model._meta.fields_map
        model_clean_fields = self._clean_model_field(model_original_fields)
        effective_field = self.get_effective_field(model_clean_fields)
        serializer_fields = BindingDict(self)

        for field_name, field_class in effective_field.items():
            current_field_class = converter.convert(self, field_class, **self.get_field_kws_by_meta(field_name))
            serializer_fields[field_name] = current_field_class
        serializer_fields.update(declared_fields)
        return serializer_fields

    def get_effective_field(self, model_fields) -> dict:
        """
        得到有效的字段
        :param model_fields: 模型字段
        :return:
        """
        meta_fields = getattr(self.Meta, 'fields', None)
        meta_exclude = getattr(self.Meta, 'exclude', None)
        if meta_exclude and meta_fields:
            raise ValueError('class ”{}“ ’Meta.fields‘ 和 ’Meta.exclude‘ 不可以共存 '.format(self.__class__.__name__))

        if meta_exclude is not None:
            return {k: v for k, v in model_fields.items() if k not in meta_exclude}
        elif meta_exclude is None and (meta_fields is None or meta_fields is ALL_FIELDS):
            return model_fields
        else:
            return {k: v for k, v in model_fields.items() if
                    (k.rsplit("_")[0] if isinstance(v, tortoise_fields.UUIDField) else k) in meta_fields}

    def get_field_kws_by_meta(self, field_name):
        """
        判断当前字段是否在Meat内设置了只读或只写
        :param field_name:
        :return:
        """
        read_only_fields = getattr(self.Meta, 'read_only_fields', [])
        write_only_fields = getattr(self.Meta, 'write_only_fields', [])
        if field_name in read_only_fields and field_name in write_only_fields:
            raise ValueError('字段 {} 不可用同时存在于类 ”{}“ 的 ’Meta.read_only_fields‘ '
                             '和 ’Meta.write_only_fields‘ 属性中'.format(field_name, self.__class__.__name__))
        if field_name in read_only_fields:
            return {'read_only': True, 'write_only': False}
        elif field_name in write_only_fields:
            return {'read_only': False, 'write_only': True}
        else:
            return {'read_only': False, 'write_only': False}

    def _clean_model_field(self, model_original_fields):
        """
        清除不需要的字段如 fk_id
        :param model_original_fields:
        :return:
        """
        clean_field_names = []
        field_dict = {}
        fields_map = copy.deepcopy(model_original_fields)
        basis_fields = self._get_model_basis_fields(fields_map)

        for field_name, field_class in basis_fields.items():
            if isinstance(field_class, (
                    tortoise_fields.relational.ForeignKeyFieldInstance,
                    tortoise_fields.relational.OneToOneFieldInstance)):
                clean_field_names.append(field_class.source_field)
            field_dict[field_name] = field_class

        for clean_field_name in clean_field_names:
            if clean_field_name in field_dict:
                field_dict.pop(clean_field_name)
        return field_dict

    async def create(self, validated_data):
        """
        根据验证后的数据进行创建，
        """
        # raise_errors_on_nested_writes('create', self, validated_data)

        ModelClass = self.Meta.model
        ModelClassMeta = ModelClass._meta
        many_to_many = {}

        for m2m_field in ModelClassMeta.m2m_fields:
            if m2m_field in self.fields:
                if m2m_field in validated_data:
                    many_to_many[m2m_field] = validated_data.pop(m2m_field)
        try:
            instance = ModelClass()
            for attr, value in validated_data.items():
                if attr not in many_to_many:
                    setattr(instance, attr, value)
            await instance.save()
        except TypeError as exc:
            raise exc
        if many_to_many:
            for field_name, values in many_to_many.items():
                field = getattr(instance, field_name)
                for value in values:
                    await field.add(value)
        return instance

    async def update(self, instance, validated_data):
        """更新"""
        ModelClass = self.Meta.model
        ModelClassMeta = ModelClass._meta

        m2m_fields = []
        for attr, value in validated_data.items():
            if attr in ModelClassMeta.m2m_fields:
                m2m_fields.append((attr, value))
            else:
                setattr(instance, attr, value)
        await instance.save()
        for attr, values in m2m_fields:
            field = getattr(instance, attr)
            for value in values:
                value, _ = await field.remote_model.get_or_create(**value)
                await field.add(value)
        return instance

    def check_relationship(self):
        """检查关系字段 目前不能为关系字段提供自动转换功能"""

    #
    def _get_model_basis_fields(self, model_fields):
        """
        得到基础字段，非关系字段
        :param model:
        :return:
        """
        return {field_name: field_class for field_name, field_class in model_fields.items() if
                not isinstance(field_class, tortoise_fields.relational.RelationalField)}

    def _get_model_M2M_fields(self, model_fields):
        """
        得到多对多字段
        :param model:
        :return:
        """
        return {field_name: field_class for field_name, field_class in model_fields.items() if
                isinstance(field_class, tortoise_fields.relational.ManyToManyFieldInstance)}

    def _get_model_O2O_fields(self, model_fields):
        """得到一对一字段"""
        return {field_name: field_class for field_name, field_class in model_fields.items() if
                isinstance(field_class, (
                    tortoise_fields.relational.BackwardOneToOneRelation,
                    tortoise_fields.relational.OneToOneFieldInstance))}

    def _get_model_M2O_fields(self, model_fields):
        """
        得到多对一字段
        :param model:
        :return:
        """
        return {field_name: field_class for field_name, field_class in model_fields.items() if
                isinstance(field_class, tortoise_fields.relational.ForeignKeyFieldInstance)}

    def _get_model_O2M_fields(self, model_fields):
        """
        得到一对多字段
        :param model:
        :return:
        """
        return {field_name: field_class for field_name, field_class in model_fields.items() if
                isinstance(field_class, tortoise_fields.relational.BackwardFKRelation)}



# -*- coding: utf-8 -*-
import collections
from collections.abc import MutableMapping


class BindingDict(MutableMapping):
    """
    这个类似于dict的对象用于在序列化器中存储字段。
    这确保了无论何时将字段添加到我们调用的序列化器中
    field.bind() 使 field_name 和 parent 属性可以正确设置。
    """

    def __init__(self, serializer):
        self.serializer = serializer
        self.fields = collections.OrderedDict()

    def __setitem__(self, key, field):
        self.fields[key] = field
        field.bind(field_name=key, parent=self.serializer)

    def __getitem__(self, key):
        return self.fields[key]

    def __delitem__(self, key):
        del self.fields[key]

    def __iter__(self):
        return iter(self.fields)

    def __len__(self):
        return len(self.fields)

    def __repr__(self):
        return dict.__repr__(self.fields)

# def set_value(instance, key, value):
#     if isinstance(value, Mapping):
#         for k, v in value.items():
#             set_value(instance, k, v)
#     setattr(instance, key, value)



from setuptools import setup, find_packages
# python setup.py sdist bdist_wheel
# twine upload dist/*
with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()
setup(
    name="srf_hdkj",
    version="0.5.6",
    author="hdkj",
    author_email="",
    description="sanic view Tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=find_packages(),
    install_requires=['ujson==5.1.0', 'openpyxl==3.0.9', 'sanic-jwt'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)
